@@ -1,6 +1,32 @@
 """
 更新内容一览
 """
+
+"""
+2023/01/09
+control_version:0.8.3
+blcoklyFile_version:2.2.11
+blockly_version:2.2.6
+1、修改了shijue0和shijue1————每次图像处理后直接修改原图并且不展示图像（需要另外展示）
+"""
+
+"""
+2023/01/09
+control_version:0.8.2
+blcoklyFile_version:2.2.11
+blockly_version:2.2.6
+1、修改了视觉库shijue0中保存图片的函数，现在支持图片保存成中文名了
+"""
+
+"""
+2022/10/26
+control_version:0.8.1
+blcoklyFile_version:2.2.11
+blockly_version:2.2.5
+1、修改了shijue1中替换背景的那个函数,如果输入的背景图不存在,就默认使用白色背景而不是报错
+"""
+
+
 """
 2022/10/26
 control_version:0.8.0
